import React, { useMemo, } from 'react';
import { isStyledComponent } from 'styled-components';
import { tv } from 'tailwind-variants';
import { useId } from '../../hooks/useId';
import { MultiComboBox, SingleComboBox } from '../ComboBox';
import { DatePicker } from '../DatePicker';
import { DropZone } from '../DropZone';
import { FaCircleExclamationIcon } from '../Icon';
import { CurrencyInput, Input } from '../Input';
import { InputFile } from '../InputFile';
import { Cluster, Stack } from '../Layout';
import { Select } from '../Select';
import { StatusLabel } from '../StatusLabel';
import { Text } from '../Text';
import { Textarea } from '../Textarea';
import { visuallyHiddenText } from '../VisuallyHiddenText/VisuallyHiddenText';
const formGroup = tv({
    slots: {
        wrapper: [
            'smarthr-ui-FormControl',
            'shr-mx-[unset] shr-border-none shr-p-[unset]',
            'disabled:shr-text-disabled',
            '[&:disabled_.smarthr-ui-FormControl-label_>_span]:shr-text-disabled',
            '[&:disabled_.smarthr-ui-FormControl-errorMessage]:shr-text-inherit',
            '[&:disabled_.smarthr-ui-FormControl-supplementaryMessage]:shr-text-inherit',
            '[&:disabled_.smarthr-ui-Input]:shr-border-default/50 [&:disabled_.smarthr-ui-Input]:shr-bg-white-darken',
        ],
        label: [
            'smarthr-ui-FormControl-label',
            // flex-item が stretch してクリッカブル領域が広がりすぎないようにする
            'shr-self-start',
            'shr-px-[unset]',
        ],
        errorList: ['shr-list-none'],
        errorIcon: ['smarthr-ui-FormControl-errorMessage', 'shr-text-danger'],
    },
});
const childrenWrapper = tv({
    variants: {
        innerMargin: {
            0: '[&&&]:shr-mt-0',
            0.25: '[&&&]:shr-mt-0.25',
            0.5: '[&&&]:shr-mt-0.5',
            0.75: '[&&&]:shr-mt-0.75',
            1: '[&&&]:shr-mt-1',
            1.25: '[&&&]:shr-mt-1.25',
            1.5: '[&&&]:shr-mt-1.5',
            2: '[&&&]:shr-mt-2',
            2.5: '[&&&]:shr-mt-2.5',
            3: '[&&&]:shr-mt-3',
            3.5: '[&&&]:shr-mt-3.5',
            4: '[&&&]:shr-mt-4',
            8: '[&&&]:shr-mt-8',
            X3S: '[&&&]:shr-mt-0.25',
            XXS: '[&&&]:shr-mt-0.5',
            XS: '[&&&]:shr-mt-1',
            S: '[&&&]:shr-mt-1.5',
            M: '[&&&]:shr-mt-2',
            L: '[&&&]:shr-mt-2.5',
            XL: '[&&&]:shr-mt-3',
            XXL: '[&&&]:shr-mt-3.5',
            X3L: '[&&&]:shr-mt-4',
        },
        isRoleGroup: {
            true: '',
            false: '',
        },
    },
    compoundVariants: [
        {
            innerMargin: undefined,
            isRoleGroup: true,
            className: '[:not([hidden])_~_&&&]:shr-mt-1',
        },
        {
            innerMargin: undefined,
            isRoleGroup: false,
            className: '[:not([hidden])_~_&&&]:shr-mt-0.5',
        },
    ],
});
export const ActualFormControl = ({ title, titleType = 'blockTitle', dangerouslyTitleHidden = false, htmlFor, labelId, innerMargin, statusLabelProps = [], helpMessage, exampleMessage, errorMessages, supplementaryMessage, as = 'div', className, children, ...props }) => {
    const managedHtmlFor = useId(htmlFor);
    const managedLabelId = useId(labelId);
    const isRoleGroup = as === 'fieldset';
    const statusLabelList = Array.isArray(statusLabelProps) ? statusLabelProps : [statusLabelProps];
    const describedbyIds = useMemo(() => Object.entries({ helpMessage, exampleMessage, supplementaryMessage, errorMessages })
        .filter(({ 1: value }) => value)
        .map(([key]) => `${managedHtmlFor}_${key}`)
        .join(' '), [helpMessage, exampleMessage, supplementaryMessage, errorMessages, managedHtmlFor]);
    const actualErrorMessages = useMemo(() => {
        if (!errorMessages) {
            return [];
        }
        return Array.isArray(errorMessages) ? errorMessages : [errorMessages];
    }, [errorMessages]);
    const { wrapperStyle, labelStyle, errorListStyle, errorIconStyle, childrenWrapperStyle } = useMemo(() => {
        const { wrapper, label, errorList, errorIcon } = formGroup();
        return {
            wrapperStyle: wrapper({ className }),
            labelStyle: label({ className: dangerouslyTitleHidden ? visuallyHiddenText() : '' }),
            errorListStyle: errorList(),
            errorIconStyle: errorIcon(),
            childrenWrapperStyle: childrenWrapper({ innerMargin, isRoleGroup }),
        };
    }, [className, dangerouslyTitleHidden, innerMargin, isRoleGroup]);
    return (React.createElement(Stack, { ...props, as: as, gap: innerMargin ?? 0.5, "aria-labelledby": isRoleGroup ? managedLabelId : undefined, "aria-describedby": isRoleGroup && describedbyIds ? describedbyIds : undefined, className: wrapperStyle },
        React.createElement(Cluster, { align: "center", htmlFor: !isRoleGroup ? managedHtmlFor : undefined, id: managedLabelId, className: labelStyle, as: isRoleGroup ? 'legend' : 'label', 
            // Stack 対象にしないための hidden
            hidden: dangerouslyTitleHidden || undefined },
            React.createElement(Text, { as: "span", styleType: titleType }, title),
            statusLabelList.length > 0 && (React.createElement(Cluster, { gap: 0.25, as: "span" }, statusLabelList.map((statusLabelProp, index) => (React.createElement(StatusLabel, { ...statusLabelProp, key: index })))))),
        helpMessage && (React.createElement("p", { className: "smarthr-ui-FormControl-helpMessage", id: `${managedHtmlFor}_helpMessage` }, helpMessage)),
        exampleMessage && (React.createElement(Text, { as: "p", color: "TEXT_GREY", italic: true, id: `${managedHtmlFor}_exampleMessage`, className: "smarthr-ui-FormControl-exampleMessage" }, exampleMessage)),
        actualErrorMessages.length > 0 && (React.createElement("div", { id: `${managedHtmlFor}_errorMessages`, className: errorListStyle, role: "alert" }, actualErrorMessages.map((message, index) => (React.createElement("p", { key: index },
            React.createElement(FaCircleExclamationIcon, { text: message, className: errorIconStyle })))))),
        React.createElement("div", { className: childrenWrapperStyle }, decorateFirstInputElement(children, {
            managedHtmlFor,
            describedbyIds,
            error: actualErrorMessages.length > 0,
        })),
        supplementaryMessage && (React.createElement(Text, { as: "p", size: "S", color: "TEXT_GREY", id: `${managedHtmlFor}_supplementaryMessage`, className: "smarthr-ui-FormControl-supplementaryMessage" }, supplementaryMessage))));
};
const decorateFirstInputElement = (children, params) => {
    const { managedHtmlFor, describedbyIds, error } = params;
    let foundFirstInput = false;
    const decorate = (targets) => React.Children.map(targets, (child) => {
        if (foundFirstInput || !React.isValidElement(child)) {
            return child;
        }
        if (!isInputElement(child)) {
            return React.cloneElement(child, {}, decorate(child.props.children));
        }
        foundFirstInput = true;
        const inputAttributes = {
            id: managedHtmlFor,
        };
        if (error) {
            inputAttributes.error = true;
        }
        if (describedbyIds !== '') {
            inputAttributes['aria-describedby'] = describedbyIds;
        }
        if (isComboBoxElement(child)) {
            return React.cloneElement(child, { inputAttributes });
        }
        else {
            return React.cloneElement(child, inputAttributes);
        }
    });
    return decorate(children);
};
/**
 * - CheckBox / RadioButton は内部に label を含むため対象外
 * - SearchInput は label を含むため対象外
 * - InputWithTooltip は領域が狭く FormControl を置けない場所での使用を想定しているため対象外
 *
 * @param node
 * @returns
 */
const isInputElement = (element) => {
    const type = isStyledComponent(element.type) ? element.type.target : element.type;
    return (type === Input ||
        type === CurrencyInput ||
        type === Textarea ||
        type === DatePicker ||
        type === Select ||
        type === SingleComboBox ||
        type === MultiComboBox ||
        type === InputFile ||
        type === DropZone);
};
const isComboBoxElement = (element) => {
    const type = isStyledComponent(element.type) ? element.type.target : element.type;
    return type === SingleComboBox || type === MultiComboBox;
};
export const FormControl = ActualFormControl;
//# sourceMappingURL=FormControl.js.map