import React, { forwardRef, useCallback, useId, useImperativeHandle, useMemo, useRef, useState, } from 'react';
import { tv } from 'tailwind-variants';
import { BaseColumn } from '../Base';
import { Button } from '../Button';
import { FaFolderOpenIcon, FaTrashAltIcon } from '../Icon';
import { Stack } from '../Layout';
const inputFile = tv({
    slots: {
        wrapper: 'smarthr-ui-InputFile shr-block',
        fileList: ['smarthr-ui-InputFile-fileList', 'shr-list-none shr-self-stretch shr-text-base'],
        fileItem: 'shr-flex shr-items-center',
        inputWrapper: [
            'shr-border-shorthand shr-relative shr-inline-flex shr-rounded-m shr-bg-white shr-font-bold shr-leading-none',
            'contrast-more:shr-border-high-contrast',
            'focus-within:shr-focus-indicator',
        ],
        input: [
            'smarthr-ui-InputFile-input',
            'shr-absolute shr-left-0 shr-top-0 shr-h-full shr-w-full shr-opacity-0',
            'file:shr-h-full file:shr-w-full file:shr-cursor-pointer',
            'file:disabled:shr-cursor-not-allowed',
        ],
        prefix: 'shr-me-0.5 shr-inline-flex',
    },
    variants: {
        size: {
            default: {
                inputWrapper: 'shr-px-1 shr-py-0.75 shr-text-base',
            },
            s: {
                inputWrapper: 'shr-p-0.5 shr-text-sm',
            },
        },
        disabled: {
            true: {
                inputWrapper: 'shr-border-default/50 shr-bg-white/50 shr-text-disabled',
            },
            false: {
                inputWrapper: 'hover:shr-border-darken hover:shr-bg-white-darken hover:shr-text-black',
            },
        },
        error: {
            true: {
                inputWrapper: '[&&&]:shr-border-danger',
            },
        },
    },
});
const DESTROY_BUTTON_TEXT = '削除';
export const InputFile = forwardRef(({ className, size = 'default', label, hasFileList = true, onChange, disabled = false, error, decorators, ...props }, ref) => {
    const [files, setFiles] = useState([]);
    const labelId = useId();
    const { wrapper, fileList, fileItem, inputWrapper, input, prefix } = inputFile();
    const wrapperStyle = useMemo(() => wrapper({ className }), [className, wrapper]);
    const inputWrapperStyle = useMemo(() => inputWrapper({ size, disabled, error }), [disabled, error, inputWrapper, size]);
    // Safari において、input.files への直接代入時に onChange が発火することを防ぐためのフラグ
    const isUpdatingFilesDirectly = useRef(false);
    const inputRef = useRef(null);
    useImperativeHandle(ref, () => inputRef.current);
    const destroyButtonText = useMemo(() => decorators?.destroy?.(DESTROY_BUTTON_TEXT) || DESTROY_BUTTON_TEXT, [decorators]);
    const updateFiles = useCallback((newFiles) => {
        onChange && onChange(newFiles);
        setFiles(newFiles);
    }, [setFiles, onChange]);
    const handleChange = useCallback((e) => {
        if (isUpdatingFilesDirectly.current) {
            return;
        }
        const newFiles = Array.from(e.target.files ?? []);
        updateFiles(newFiles);
    }, [isUpdatingFilesDirectly, updateFiles]);
    const handleDelete = useCallback((index) => {
        if (!inputRef.current) {
            return;
        }
        const newFiles = files.filter((_, i) => index !== i);
        updateFiles(newFiles);
        const buff = new DataTransfer();
        newFiles.forEach((file) => {
            buff.items.add(file);
        });
        isUpdatingFilesDirectly.current = true;
        inputRef.current.files = buff.files;
        isUpdatingFilesDirectly.current = false;
    }, [files, isUpdatingFilesDirectly, inputRef, updateFiles]);
    return (React.createElement(Stack, { align: "flex-start", className: wrapperStyle },
        !disabled && hasFileList && files.length > 0 && (React.createElement(BaseColumn, { as: "ul", padding: { block: 0.5, inline: 1 }, className: fileList() }, files.map((file, index) => (React.createElement("li", { key: `${file.name}-${index}`, className: fileItem() },
            React.createElement("span", { className: "smarthr-ui-InputFile-fileName" }, file.name),
            React.createElement(Button, { variant: "text", prefix: React.createElement(FaTrashAltIcon, null), onClick: () => handleDelete(index), className: "smarthr-ui-InputFile-deleteButton" }, destroyButtonText)))))),
        React.createElement("span", { className: inputWrapperStyle },
            React.createElement("input", { ...props, type: "file", onChange: handleChange, disabled: disabled, className: input(), ref: inputRef, "aria-invalid": error || undefined, "aria-labelledby": labelId }),
            React.createElement("span", { className: prefix() },
                React.createElement(FaFolderOpenIcon, null)),
            React.createElement("span", { id: labelId, "aria-hidden": "true" }, label))));
});
//# sourceMappingURL=InputFile.js.map