import React, { useRef, useState, useSyncExternalStore, } from 'react';
import { createPortal } from 'react-dom';
import { tv } from 'tailwind-variants';
import { useEnhancedEffect } from '../../hooks/useEnhancedEffect';
import { useId } from '../../hooks/useId';
import { TooltipPortal } from './TooltipPortal';
const subscribeFullscreenChange = (callback) => {
    window.addEventListener('fullscreenchange', callback);
    return () => {
        window.removeEventListener('fullscreenchange', callback);
    };
};
const getFullscreenElement = () => document.fullscreenElement;
const getFullscreenElementOnSSR = () => null;
const tooltip = tv({
    base: [
        'smarthr-ui-Tooltip',
        'shr-inline-block shr-max-w-full shr-overflow-y-hidden',
        /* inline-block に overflow: visible 以外を指定すると、vertical-align が bottom margin edge に揃ってしまう
         * https://ja.stackoverflow.com/questions/2603/ */
        'shr-align-bottom',
        'focus-visible:shr-focus-indicator',
    ],
    variants: {
        isIcon: {
            true: 'shr-leading-[0]',
        },
    },
});
export const Tooltip = ({ message, children, triggerType, multiLine, ellipsisOnly = false, horizontal = 'left', vertical = 'bottom', tabIndex = 0, ariaDescribedbyTarget = 'wrapper', className, onPointerEnter, onPointerLeave, onTouchStart, onTouchEnd, onFocus, onBlur, ...props }) => {
    const [portalRoot, setPortalRoot] = useState(null);
    const [isVisible, setIsVisible] = useState(false);
    const [rect, setRect] = useState(null);
    const ref = useRef(null);
    const tooltipId = useId();
    const fullscreenElement = useSyncExternalStore(subscribeFullscreenChange, getFullscreenElement, getFullscreenElementOnSSR);
    useEnhancedEffect(() => {
        setPortalRoot(fullscreenElement ?? document.body);
    }, [fullscreenElement]);
    const getHandlerToShow = (handler) => (e) => {
        handler && handler(e);
        if (!ref.current) {
            return;
        }
        if (ellipsisOnly) {
            const outerWidth = parseInt(window
                .getComputedStyle(ref.current.parentNode, null)
                .width.match(/\d+/)[0], 10);
            const wrapperWidth = ref.current.clientWidth;
            const existsEllipsis = outerWidth >= 0 && outerWidth <= wrapperWidth;
            if (!existsEllipsis) {
                return;
            }
        }
        setRect(ref.current.getBoundingClientRect());
        setIsVisible(true);
    };
    const getHandlerToHide = (handler) => (e) => {
        handler && handler(e);
        setIsVisible(false);
    };
    const isIcon = triggerType === 'icon';
    const styles = tooltip({ isIcon, className });
    const childrenWithProps = ariaDescribedbyTarget === 'inner'
        ? React.cloneElement(children, { 'aria-describedby': tooltipId })
        : children;
    return (
    // eslint-disable-next-line jsx-a11y/no-static-element-interactions,smarthr/a11y-delegate-element-has-role-presentation
    React.createElement("span", { ...props, "aria-describedby": ariaDescribedbyTarget === 'wrapper' ? tooltipId : undefined, ref: ref, onPointerEnter: getHandlerToShow(onPointerEnter), onTouchStart: getHandlerToShow(onTouchStart), onFocus: getHandlerToShow(onFocus), onPointerLeave: getHandlerToHide(onPointerLeave), onTouchEnd: getHandlerToHide(onTouchEnd), onBlur: getHandlerToHide(onBlur), tabIndex: tabIndex, className: styles },
        portalRoot &&
            createPortal(React.createElement(TooltipPortal, { message: message, id: tooltipId, isVisible: isVisible, parentRect: rect, isIcon: isIcon, isMultiLine: multiLine, horizontal: horizontal, vertical: vertical, fullscreenElement: fullscreenElement }), portalRoot),
        childrenWithProps));
};
//# sourceMappingURL=Tooltip.js.map